import React, { useEffect } from 'react';
import { AppContext, AppEventType, RouterEventType } from 'react-native-stack';
import { useDispatch, useStore } from 'react-redux';
import { routerLocationChange } from '../actions';
import { appStateKey } from '../index';
import { rehydrate as rehydrateApp } from '../rehydrate';
function addRouterListener(app, key, dispatch) {
    return app.routers[key].addEventListener(event => {
        switch (event.type) {
            case RouterEventType.LocationChange:
                dispatch(routerLocationChange(key, event.action, event.index, event.entries, event.length, event.location));
                break;
            case RouterEventType.LocationStateChange:
                dispatch(routerLocationChange(key, app.routers[key].action, app.routers[key].index, app.routers[key].entries, app.routers[key].length, app.routers[key].location));
                break;
        }
    });
}
export const ConnectedAppContainer = ({ app, children, rehydrate }) => {
    const dispatch = useDispatch();
    useEffect(() => {
        const unregisterCallbacks = [];
        const routersUnregisterCallbacks = {};
        if (app) {
            for (const key in app.routers) {
                if (app.routers.hasOwnProperty(key)) {
                    routersUnregisterCallbacks[key] = addRouterListener(app, key, dispatch);
                }
            }
            unregisterCallbacks.push(app.addEventListener(event => {
                switch (event.type) {
                    case AppEventType.RouterAdd:
                        routersUnregisterCallbacks[event.key] = addRouterListener(app, event.key, dispatch);
                        break;
                    case AppEventType.RouterRemove:
                        if (routersUnregisterCallbacks[event.key]) {
                            routersUnregisterCallbacks[event.key]();
                        }
                        break;
                }
            }));
        }
        return () => {
            for (const key in routersUnregisterCallbacks) {
                if (routersUnregisterCallbacks.hasOwnProperty(key) &&
                    routersUnregisterCallbacks[key]) {
                    routersUnregisterCallbacks[key]();
                }
            }
            unregisterCallbacks.forEach(c => c());
        };
    }, []);
    const store = useStore();
    if (app && rehydrate) {
        const state = store.getState();
        if (state[appStateKey]) {
            rehydrateApp(app, state[appStateKey]);
        }
    }
    return <AppContext.Provider value={app} children={children}/>;
};
